@@ -2,19 +2,19 @@
 
 # Copyright 2010-2013 Joakim Möller
 # Copyright 2014 Marek Rudnicki
-
+#
 # This file is part of matlab_wrapper.
-
+#
 # matlab_wrapper is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
-
+#
 # matlab_wrapper is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-
+#
 # You should have received a copy of the GNU General Public License
 # along with matlab_wrapper.  If not, see <http://www.gnu.org/licenses/>.
 
@@ -167,6 +167,8 @@ class MatlabSession(object):
 
         error_string = self._libmx.mxArrayToString(mxresult)
 
+        self._libmx.mxDestroyArray(mxresult)
+
         if error_string != "":
             raise RuntimeError("Error from MATLAB\n{0}".format(error_string))
 
@@ -428,7 +430,7 @@ class Workspace(object):
             out = session.get(attr)
 
         elif kind in (2, 3, 5, 6): # Function
-            out = MatlabFunction(name=attr, session=session)
+            out = MatlabFunction(name=attr, session_ref=self._session_ref)
 
         else:
             raise NotImplementedError("Unknown variable/function type in MATLAB workspace: {}".format(attr))
@@ -447,12 +449,13 @@ class Workspace(object):
 
 
 class MatlabFunction(object):
-    def __init__(self, name, session):
+    def __init__(self, name, session_ref):
         self.name = name
-        self._session = session
+        self._session_ref = session_ref
+
 
     def __call__(self, *args, **kwargs):
-        session = self._session
+        session = self._session_ref()
 
 
         ### Left-hand side (returns) string
@@ -516,7 +519,7 @@ class MatlabFunction(object):
     @property
     def __doc__(self):
 
-        session = self._session
+        session = self._session_ref()
 
         session.eval(
             "DOC__ = help('{}')".format(self.name)
